import java.awt.*;
import java.awt.event.*;

public class ColorSliderDialog extends Dialog
{	/** Create a new color slider dialog.  The dialog will accept an rgb color from
		the user and when the "Do It" button is pressed the color of the component named
		in the initialization will be changed to this new color.
	*/
	public ColorSliderDialog(Frame frame, Component component)
	{	super(frame, "Color Choice",false);
		this.component = component;
		setSize(225, 100);
		setResizable(false);
		add("Center", dialogPanel);
		dialogPanel.setLayout(new GridLayout(3, 2, 0, 2));
		dialogPanel.add(redLabel);
		dialogPanel.add(red);
		red.setValues(0, 0, 0, 255);
		dialogPanel.add(greenLabel);
		dialogPanel.add(green);
		green.setValues(0, 0, 0, 255);
		dialogPanel.add(blueLabel);
		dialogPanel.add(blue);	
		blue.setValues(0, 0, 0, 255);
		Panel buttonPanel = new Panel(); // Layout the buttons.  
		add("South", buttonPanel);
		buttonPanel.add( new YesButton());
		buttonPanel.add( new NoButton());	
		add("East", colorPanel);
		colorPanel.setBackground(Color.black);
	}
	
	public Color getColor()
	{	return currentColor;
	}
	
	ColorPanel colorPanel = new ColorPanel();
	Panel dialogPanel = new Panel(); // Layout the fields and labels
	Scrollbar red = new ColorScroller();
	Scrollbar green = new ColorScroller();
	Scrollbar blue = new ColorScroller();
	Label redLabel = new Label("Red 0");
	Label greenLabel = new Label ("Green 0");
	Label blueLabel = new Label ("Blue 0");
	Component component;
	Color currentColor = Color.black;
		
	private class ColorPanel extends Panel
	{	public Dimension getMinimumSize()
		{	return new Dimension(25, 50);
		}
	}
	
	private class YesButton extends Button implements ActionListener // Yes button for UserColor dialog.
	{	public YesButton()
		{	super( "Do It");
			addActionListener(this);
		}
		
		public void actionPerformed(ActionEvent e)
		{	int r = red.getValue();
			int g = green.getValue();
			int b = blue.getValue();
			redLabel.setText("Red " + r);
			greenLabel.setText("Green " + g);
			blueLabel.setText("Blue " + b);
			currentColor = new Color(r, g, b);
			if(component != null)
			{	component.setBackground(currentColor);
				component.repaint();
			}
		}
	}

	private class NoButton extends Button implements ActionListener // No button for UserColor dialog.
	{	public NoButton()
		{	super( "Cancel");
			addActionListener(this);
		}
		
		public void actionPerformed(ActionEvent e)
		{	ColorSliderDialog.this.setVisible(false);
		}
	}
	
	private class ColorScroller extends Scrollbar implements AdjustmentListener
	{	public ColorScroller()
		{	super(Scrollbar.HORIZONTAL);
			addAdjustmentListener(this);
		}
	
		public void adjustmentValueChanged(AdjustmentEvent e)
		{	int r = red.getValue();
			int g = green.getValue();
			int b = blue.getValue();
			redLabel.setText("Red " + r);
			greenLabel.setText("Green " + g);
			blueLabel.setText("Blue " + b);
			colorPanel.setBackground(new Color(r, g, b));
			colorPanel.repaint();
		}
	}
	
}