import java.awt.*;

public abstract class Figure implements java.io.Serializable
{	protected Rectangle bounds;
	protected String label;
	protected boolean selected = false;
	
	public Figure(String label, int x, int y, int w, int h)
	{	bounds = new Rectangle(x, y, w, h);
		this.label = label;
	}
	
	public abstract void draw(Graphics g);
	public abstract void drawOutline(Graphics g);
	
	public Connection connect(String label, Figure b)
	{	return new Connection(label, this, b);
	}
	
	public boolean isSelected(){ return selected;}
	public void setSelected(boolean t){selected = t;}
	public void toggleSelected(){selected = !selected;}
	
	public void drawSelectors(Graphics g)
	{	g.fillRect(bounds.x - 2 , bounds.y - 2, 4, 4);
		g.fillRect(bounds.x + bounds.width - 2 , bounds.y-2, 4, 4);
		g.fillRect(bounds.x-2 , bounds.y + bounds.height - 2, 4, 4);
		g.fillRect(bounds.x + bounds.width - 2 , bounds.y + bounds.height - 2, 4, 4);
	}
	
	public Rectangle rectangle(){ return bounds; }
	public Point location(){ return new Point(bounds.x, bounds.y); }
	public Point center(){ return new Point(bounds.x+bounds.width/2, bounds.y+bounds.height/2); }
	public Dimension dimension(){ return new Dimension(bounds.width, bounds.height);}
	
	public void move(int x, int y)
	{	bounds.x = x; bounds.y = y;
	}
	
	public void move(Point p)
	{	bounds.x = p.x; bounds.y = p.y;
	}
	
	public boolean contains(Point p)
	{	return bounds.contains(p);
	}
	
	public boolean contains(int x, int y)
	{	return bounds.contains(x, y);
	}
	
}

